#if !defined( __LIBFISHEYE_H__ )
#define __LIBFISHEYE_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MAKEFOURCC
    #define MAKEFOURCC(ch0, ch1, ch2, ch3)                          \
                ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) |   \
                ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 ))
#endif //defined(MAKEFOURCC)


#define LIBFISHEYE_VERSION MAKEFOURCC('2', '0', '0', '0')

#ifndef DLLAPI
#ifdef _WIN32
#ifdef _USRDLL
#define DLLAPI __declspec(dllexport) __stdcall
#else
#define DLLAPI __stdcall
#endif //defined(_USRDLL)
#else
#define DLLAPI
#endif //defined(_WIN32)
#endif //defined(DLLAPI)

#include "typedef.h"

typedef enum _FisheyeVPixelFormat
{
	FE_PIXELFORMAT_YUV420P,  // YUV420 planar
	FE_PIXELFORMAT_RGB32     // RGBA interleaved
}FEVPIXELFORMAT;

typedef enum _FisheyeMountType
{
	FE_MOUNT_WALL,     // Wall
	FE_MOUNT_CEILING,  // Ceiling 
	FE_MOUNT_FLOOR     // Floor
}FEMOUNTTYPE;

typedef enum _FisheyeDewarpType
{
	FE_DEWARP_RECTILINEAR,       // Rectilinear
	FE_DEWARP_FULLVIEWPANORAMA,  // Full-view panorama
	FE_DEWARP_DUALVIEWPANORAMA,  // Dual-view panorama (which is only available in FE_MOUNT_CEILING and FE_MOUNT_CEILING type)
	FE_DEWARP_CLIPVIEWPANORAMA   // Full-HD panorama
}FEDEWARPTYPE;

typedef enum _FisheyeLensType
{
	FE_LENSTYPE1,	// FE8171
	FE_LENSTYPE2,	// FE8172
	FE_LENSTYPE3,	// FE8173
	FE_LENSTYPE4,   // FE8174 , FE8181
	FE_LENSTYPE5,	// VC8201
	FE_LENSTYPE6	// FE8180
}FELENSTYPE;

typedef enum _FisheyeOptionFlag
{
	FE_OPTION_INIMAGEHEADER		= (1 << 0),   // Input image header
	FE_OPTION_INIMAGEBUFFER		= (1 << 1),   // Input image buffer
	FE_OPTION_OUTIMAGEHEADER	= (1 << 2),   // Output image header
	FE_OPTION_OUTIMAGEBUFFER	= (1 << 3),   // Output image buffer
	FE_OPTION_FOVCENTER			= (1 << 4),   // FOV's center
	FE_OPTION_FOVRADIUS			= (1 << 5),   // FOV's radius
	FE_OPTION_MOUNTTYPE			= (1 << 6),   // Mount type (which is one of the FEMOUNTTYPE)
	FE_OPTION_DEWARPTYPE		= (1 << 7),   // Dewarp type (which is one of the FEDEWARPTYPE)
	FE_OPTION_YAW				= (1 << 8),   // Yaw
	FE_OPTION_PITCH				= (1 << 9),   // Pitch
	FE_OPTION_ROLL				= (1 << 10),  // Roll
	FE_OPTION_SCALE				= (1 << 11),  // Scale
	FE_OPTION_SCROLL			= (1 << 12),  // Scroll
	FE_OPTION_OUTROI			= (1 << 13),  // Ouput ROI
	FE_OPTION_LENSTYPE			= (1 << 14)	  // Lens
}FEOPTIONFLAG;

typedef struct _FisheyePoint
{
	int X;  // X
	int Y;  // Y
}FEPOINT;

typedef struct _FisheyeRect
{
	int Left;    // Left
	int Top;     // Top
	int Right;   // Right
	int Bottom;  // Bottom
}FERECT;

typedef struct _FisheyeVPicture
{
	/* Header field */
	unsigned int   Width;    // Width
	unsigned int   Height;   // Height
	unsigned int   Stride;   // Stride
	FEVPIXELFORMAT Format;   // Pixel format (which is one of the FEVPIXELFORMAT)

	/* Buffer field */
	BYTE           *Buffer;  // Buffer
}FEVPICTURE;

typedef struct _FisheyeOption
{
	DWORD        Flags;        // Option flags (which is the combination of one or more flags in FEOPTIONFLAG)
	FEVPICTURE   InVPicture;   // Input picture
	FEVPICTURE   OutVPicture;  // Output picture
	FEPOINT      FOVCenter;    // FOV's center
	unsigned int FOVRadius;    // FOV's radius
	FEMOUNTTYPE  MountType;    // Mount type 
	FEDEWARPTYPE DewarpType;   // Dewarp type
	float        Yaw;          // Yaw (rotate around the y-axis)
	float        Pitch;        // Pitch (rotate around the x-axis)
	float        Roll;         // Roll (rotate around the z-axis)
	unsigned int Scale;        // Scaling factor
	float        Scroll;       // Scroll
	FERECT       OutRoi;       // Output ROI
	FELENSTYPE	 LensType;	   // Lens Type
}FEOPTION;

/* Initial the fisheye dewarp module */
SCODE DLLAPI Fisheye_Initial(HANDLE *phObject, DWORD dwVersion);

/* Release the fisheye dewarp module */
SCODE DLLAPI Fisheye_Release(HANDLE *phObject);

/* Dewarp the fisheye frame */
SCODE DLLAPI Fisheye_OneFrame(HANDLE hObject);

/* Change the settings in fisheye dewarp module */
SCODE DLLAPI Fisheye_SetOption(HANDLE hObject, FEOPTION *pOption);

/* Get the Yaw, Pitch and Roll angles from the source fisheye frame */
void DLLAPI Fisheye_PositionToYawPitchRoll(FEPOINT FOVCenter, int FOVRadius, FEMOUNTTYPE MountType, int X, int Y, float *pYaw, float *pPitch, float *pRoll);


#ifdef __cplusplus
}
#endif

#endif  // __LIBFISHEYE_H__